@@ -10,10 +10,9 @@ to be different but are now unified except for some fossilized names.) */
 #include "m_pd.h"
 #include "m_imp.h"
 #include "s_stuff.h"
-#include "s_utf8.h"
 #include "g_canvas.h"
+#include "s_utf8.h"
 #include <string.h>
-#include "g_all_guis.h"
 #include "g_undo.h"
 
 #ifdef _WIN32
@@ -76,7 +75,7 @@ void gobj_displace(t_gobj *x, t_glist *glist, int dx, int dy)
 
     /* here we add an extra check whether we're mapped, because some
        editing moves are carried out on invisible windows (notably, re-creating
-       abstractions when one is saved).  Should any other widget finctions also
+       abstractions when one is saved).  Should any other widget functions also
        be doing this?  */
 void gobj_select(t_gobj *x, t_glist *glist, int state)
 {
@@ -165,7 +164,7 @@ void canvas_updatewindowlist(void)
 {
             /* not if we're in a reload */
     if (!THISGUI->i_reloadingabstraction)
-        sys_gui("::pd_menus::update_window_menu\n");
+        pdgui_vmess("::pd_menus::update_window_menu", 0);
 }
 
     /* add a glist the list of "root" canvases (toplevels without parents.) */
@@ -740,8 +739,10 @@ void canvas_reflecttitle(t_canvas *x)
         strncat(namebuf, " [edit]", MAXPDSTRING-strlen(namebuf)-1);
         namebuf[MAXPDSTRING-1] = 0;
     }
-    sys_vgui("pdtk_canvas_reflecttitle .x%lx {%s} {%s} {%s} %d\n",
-        x, canvas_getdir(x)->s_name, x->gl_name->s_name, namebuf, x->gl_dirty);
+    pdgui_vmess("pdtk_canvas_reflecttitle", "^ sss i",
+        x,
+        canvas_getdir(x)->s_name, x->gl_name->s_name, namebuf,
+        x->gl_dirty);
 }
 
     /* mark a glist dirty or clean */
@@ -769,12 +770,16 @@ void canvas_drawredrect(t_canvas *x, int doit)
             x2 = x1 + x->gl_zoom * x->gl_pixwidth,
             y1 = x->gl_zoom * x->gl_ymargin,
             y2 = y1 + x->gl_zoom * x->gl_pixheight;
-        sys_vgui(".x%lx.c create line %d %d %d %d %d %d %d %d %d %d "
-            "-fill #ff8080 -width %d -capstyle projecting -tags GOP\n",
-            glist_getcanvas(x), x1, y1, x1, y2, x2, y2, x2, y1, x1, y1,
-                x->gl_zoom);
+        pdgui_vmess(0, "crr iiiiiiiiii rr ri rr rr",
+            glist_getcanvas(x), "create", "line",
+            x1,y1, x1,y2, x2,y2, x2,y1, x1,y1,
+            "-fill", "#ff8080",
+            "-width", x->gl_zoom,
+            "-capstyle", "projecting",
+            "-tags", "GOP"); /* better: "-tags", 1, &"GOP" */
     }
-    else sys_vgui(".x%lx.c delete GOP\n",  glist_getcanvas(x));
+    else
+        pdgui_vmess(0, "crs", glist_getcanvas(x), "delete", "GOP");
 }
 
     /* the window becomes "mapped" (visible and not miniaturized) or
@@ -802,7 +807,7 @@ void canvas_map(t_canvas *x, t_floatarg f)
             canvas_drawlines(x);
             if (x->gl_isgraph && x->gl_goprect)
                 canvas_drawredrect(x, 1);
-            sys_vgui("pdtk_canvas_getscroll .x%lx.c\n", x);
+            pdgui_vmess("pdtk_canvas_getscroll", "c", x);
         }
     }
     else
@@ -815,7 +820,7 @@ void canvas_map(t_canvas *x, t_floatarg f)
                 return;
             }
                 /* just clear out the whole canvas */
-            sys_vgui(".x%lx.c delete all\n", x);
+            pdgui_vmess(0, "crs", x, "delete", "all");
             x->gl_mapped = 0;
         }
     }
@@ -922,7 +927,7 @@ void canvas_free(t_canvas *x)
     freebytes(x->gl_xlabel, x->gl_nxlabels * sizeof(*(x->gl_xlabel)));
     freebytes(x->gl_ylabel, x->gl_nylabels * sizeof(*(x->gl_ylabel)));
     gstub_cutoff(x->gl_stub);
-    gfxstub_deleteforkey(x);        /* probably unnecessary */
+    pdgui_stub_deleteforkey(x);        /* probably unnecessary */
     if (!x->gl_owner && !x->gl_isclone)
         canvas_takeofflist(x);
 }
@@ -934,34 +939,48 @@ static void canvas_drawlines(t_canvas *x)
     t_linetraverser t;
     t_outconnect *oc;
     {
+        char tag[128];
+        const char*tags[2] = {tag, "cord"};
         linetraverser_start(&t, x);
         while ((oc = linetraverser_next(&t)))
-            sys_vgui(
-        ".x%lx.c create line %d %d %d %d -width %d -tags [list l%lx cord]\n",
-                glist_getcanvas(x),
-                t.tr_lx1, t.tr_ly1, t.tr_lx2, t.tr_ly2,
-                (outlet_getsymbol(t.tr_outlet) == &s_signal ? 2:1) * x->gl_zoom,
-                oc);
+        {
+            sprintf(tag, "l%lx", oc);
+            pdgui_vmess(0, "crr iiii ri rS",
+                glist_getcanvas(x), "create", "line",
+                t.tr_lx1,t.tr_ly1, t.tr_lx2,t.tr_ly2,
+                "-width", (outlet_getsymbol(t.tr_outlet) == &s_signal ? 2:1) * x->gl_zoom,
+                "-tags", 2, tags);
+        }
     }
 }
-
 void canvas_fixlinesfor(t_canvas *x, t_text *text)
 {
     t_linetraverser t;
     t_outconnect *oc;
-    
+
     linetraverser_start(&t, x);
     while ((oc = linetraverser_next(&t)))
     {
         if (t.tr_ob == text || t.tr_ob2 == text)
         {
-            sys_vgui(".x%lx.c coords l%lx %d %d %d %d\n",
-                glist_getcanvas(x), oc,
-                t.tr_lx1, t.tr_ly1, t.tr_lx2, t.tr_ly2);
+            char tag[128];
+            sprintf(tag, "l%lx", oc);
+            pdgui_vmess(0, "crs iiii",
+                glist_getcanvas(x), "coords", tag,
+                t.tr_lx1,t.tr_ly1, t.tr_lx2,t.tr_ly2);
         }
     }
 }
 
+static void _canvas_delete_line(t_canvas*x, t_outconnect *oc)
+{
+    char tag[128];
+    if (!glist_isvisible(x))
+        return;
+    sprintf(tag, "l%lx", oc);
+    pdgui_vmess(0, "crs", glist_getcanvas(x), "delete", tag);
+}
+
     /* kill all lines for the object */
 void canvas_deletelinesfor(t_canvas *x, t_text *text)
 {
@@ -972,11 +991,7 @@ void canvas_deletelinesfor(t_canvas *x, t_text *text)
     {
         if (t.tr_ob == text || t.tr_ob2 == text)
         {
-            if (glist_isvisible(x))
-            {
-                sys_vgui(".x%lx.c delete l%lx\n",
-                    glist_getcanvas(x), oc);
-            }
+            _canvas_delete_line(x, oc);
             obj_disconnect(t.tr_ob, t.tr_outno, t.tr_ob2, t.tr_inno);
         }
     }
@@ -994,11 +1009,7 @@ void canvas_deletelinesforio(t_canvas *x, t_text *text,
         if ((t.tr_ob == text && t.tr_outlet == outp) ||
             (t.tr_ob2 == text && t.tr_inlet == inp))
         {
-            if (glist_isvisible(x))
-            {
-                sys_vgui(".x%lx.c delete l%lx\n",
-                    glist_getcanvas(x), oc);
-            }
+            _canvas_delete_line(x, oc);
             obj_disconnect(t.tr_ob, t.tr_outno, t.tr_ob2, t.tr_inno);
         }
     }
@@ -1372,7 +1383,7 @@ static void canvas_start_dsp(void)
 {
     t_canvas *x;
     if (THISGUI->i_dspstate) ugen_stop();
-    else sys_gui("pdtk_pd_dsp ON\n");
+    else pdgui_vmess("pdtk_pd_dsp", "s", "ON");
     ugen_start();
 
     for (x = pd_getcanvaslist(); x; x = x->gl_next)
@@ -1388,7 +1399,7 @@ static void canvas_stop_dsp(void)
     if (THISGUI->i_dspstate)
     {
         ugen_stop();
-        sys_gui("pdtk_pd_dsp OFF\n");
+        pdgui_vmess("pdtk_pd_dsp", "s", "OFF");
         canvas_dspstate = THISGUI->i_dspstate = 0;
         if (gensym("pd-dsp-stopped")->s_thing)
             pd_bang(gensym("pd-dsp-stopped")->s_thing);
@@ -1418,9 +1429,9 @@ void canvas_update_dsp(void)
     if (THISGUI->i_dspstate) canvas_start_dsp();
 }
 
-/* the "dsp" message to pd starts and stops DSP somputation, and, if
+/* the "dsp" message to pd starts and stops DSP computation, and, if
 appropriate, also opens and closes the audio device.  On exclusive-access
-APIs such as ALSA, MMIO, and ASIO (I think) it\s appropriate to close the
+APIs such as ALSA, MMIO, and ASIO (I think) it's appropriate to close the
 audio devices when not using them; but jack behaves better if audio I/O
 simply keeps running.  This is wasteful of CPU cycles but we do it anyway
 and can perhaps regard this is a design flaw in jack that we're working around
@@ -2213,5 +2224,5 @@ void glob_open(t_pd *ignore, t_symbol *name, t_symbol *dir, t_floatarg f)
         return;
     }
     if (!glob_evalfile(ignore, name, dir))
-        sys_vgui("::pdwindow::busyrelease\n");
+        pdgui_vmess("::pdwindow::busyrelease", 0);
 }
