@@ -7,19 +7,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <ctype.h>
 #include "m_pd.h"
-#include "g_canvas.h"
 
 #include "g_all_guis.h"
 #include <math.h>
 
-#ifdef _WIN32
-#include <io.h>
-#else
-#include <unistd.h>
-#endif
-
 #define MINDIGITS 1
 #define MINFONT   4
 
@@ -28,27 +20,6 @@
 static void my_numbox_key(void *z, t_symbol *keysym, t_floatarg fkey);
 static void my_numbox_draw_update(t_gobj *client, t_glist *glist);
 
-/* ------------ nbx gui-my number box ----------------------- */
-
-t_widgetbehavior my_numbox_widgetbehavior;
-static t_class *my_numbox_class;
-
-/* widget helper functions */
-
-static void my_numbox_tick_reset(t_my_numbox *x)
-{
-    if(x->x_gui.x_fsf.x_change && x->x_gui.x_glist)
-    {
-        x->x_gui.x_fsf.x_change = 0;
-        sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
-    }
-}
-
-static void my_numbox_tick_wait(t_my_numbox *x)
-{
-    sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
-}
-
 void my_numbox_clip(t_my_numbox *x)
 {
     if(x->x_val < x->x_min)
@@ -128,11 +99,143 @@ void my_numbox_ftoa(t_my_numbox *x)
     }
 }
 
+/* ------------ nbx gui-my number box ----------------------- */
+
+t_widgetbehavior my_numbox_widgetbehavior;
+static t_class *my_numbox_class;
+
+#define my_numbox_draw_io 0
+
+static void my_numbox_draw_config(t_my_numbox* x, t_glist* glist)
+{
+    const int zoom = IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
+    t_iemgui *iemgui = &x->x_gui;
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int w = x->x_gui.x_w, half = x->x_gui.x_h/2;
+    int d = zoom + x->x_gui.x_h/(34*zoom);
+    int corner = x->x_gui.x_h/4;
+    int iow = IOWIDTH * zoom, ioh = IEM_GUI_IOHEIGHT * zoom;
+    char tag[128];
+
+    int lcol = x->x_gui.x_lcol;
+    int fcol = x->x_gui.x_fcol;
+    t_atom fontatoms[3];
+    SETSYMBOL(fontatoms+0, gensym(iemgui->x_font));
+    SETFLOAT (fontatoms+1, -iemgui->x_fontsize*zoom);
+    SETSYMBOL(fontatoms+2, gensym(sys_fontweight));
+
+    if(x->x_gui.x_fsf.x_selected)
+        fcol = lcol = IEM_GUI_COLOR_SELECTED;
+    if(x->x_gui.x_fsf.x_change)
+        fcol =  IEM_GUI_COLOR_EDITED;
+
+    my_numbox_ftoa(x);
+
+    sprintf(tag, "%lxBASE1", x);
+    pdgui_vmess(0, "crs  ii ii ii ii ii ii", canvas, "coords", tag,
+        xpos,              ypos,
+        xpos + w - corner, ypos,
+        xpos + w,          ypos + corner,
+        xpos + w,          ypos + x->x_gui.x_h,
+        xpos,              ypos + x->x_gui.x_h,
+        xpos,              ypos);
+    pdgui_vmess(0, "crs  ri rk rk", canvas, "itemconfigure", tag,
+        "-width", zoom,
+        "-outline", IEM_GUI_COLOR_NORMAL,
+        "-fill", x->x_gui.x_bcol);
+
+
+    sprintf(tag, "%lxBASE2", x);
+    pdgui_vmess(0, "crs  ii ii ii", canvas, "coords", tag,
+        xpos + zoom, ypos + zoom,
+        xpos + half, ypos + half,
+        xpos + zoom, ypos + x->x_gui.x_h - zoom);
+    pdgui_vmess(0, "crs  ri rk", canvas, "itemconfigure", tag,
+        "-width", zoom,
+        "-fill", x->x_gui.x_fcol);
+
+    sprintf(tag, "%lxLABEL", x);
+    pdgui_vmess(0, "crs  ii", canvas, "coords", tag,
+        xpos + x->x_gui.x_ldx * zoom,
+        ypos + x->x_gui.x_ldy * zoom);
+    pdgui_vmess(0, "crs  rA rk", canvas, "itemconfigure", tag,
+        "-font", 3, fontatoms,
+        "-fill", lcol);
+    iemgui_dolabel(x, &x->x_gui, x->x_gui.x_lab, 1);
+
+    sprintf(tag, "%lxNUMBER", x);
+    pdgui_vmess(0, "crs  ii", canvas, "coords", tag,
+        xpos + half + 2*zoom, ypos + half + d);
+    pdgui_vmess(0, "crs  rs rA rk", canvas, "itemconfigure", tag,
+        "-text", x->x_buf,
+        "-font", 3, fontatoms,
+        "-fill", fcol);
+}
+
+static void my_numbox_draw_new(t_my_numbox *x, t_glist *glist)
+{
+    t_canvas *canvas = glist_getcanvas(glist);
+    char tag[128], tag_object[128];
+    char*tags[] = {tag_object, tag, "label", "text"};
+    sprintf(tag_object, "%lxOBJ", x);
+
+    sprintf(tag, "%lxBASE1", x);
+    pdgui_vmess(0, "crr ii rS", canvas, "create", "polygon",
+        0, 0, "-tags", 2, tags);
+
+    sprintf(tag, "%lxBASE2", x);
+    pdgui_vmess(0, "crr iiii rS", canvas, "create", "line",
+        0, 0, 0, 0, "-tags", 2, tags);
+
+    sprintf(tag, "%lxLABEL", x);
+    pdgui_vmess(0, "crr ii rs rS", canvas, "create", "text",
+        0, 0, "-anchor", "w", "-tags", 4, tags);
+
+    sprintf(tag, "%lxNUMBER", x);
+    pdgui_vmess(0, "crr ii rs rS", canvas, "create", "text",
+        0, 0, "-anchor", "w", "-tags", 2, tags);
+
+    my_numbox_draw_config(x, glist);
+    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO);
+}
+
+static void my_numbox_draw_select(t_my_numbox *x, t_glist *glist)
+{
+    t_canvas *canvas = glist_getcanvas(glist);
+    int bcol = IEM_GUI_COLOR_NORMAL, lcol = x->x_gui.x_lcol, fcol = x->x_gui.x_fcol;
+    char tag[128];
+
+    if(x->x_gui.x_fsf.x_selected)
+    {
+        if(x->x_gui.x_fsf.x_change)
+        {
+            x->x_gui.x_fsf.x_change = 0;
+            x->x_buf[0] = 0;
+            sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
+        }
+        bcol = lcol = fcol = IEM_GUI_COLOR_SELECTED;
+    }
+
+    sprintf(tag, "%lxBASE1", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag, "-outline", bcol);
+    sprintf(tag, "%lxBASE2", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag, "-fill", fcol);
+    sprintf(tag, "%lxLABEL", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag, "-fill", lcol);
+    sprintf(tag, "%lxNUMBER", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag, "-fill", fcol);
+}
+
 static void my_numbox_draw_update(t_gobj *client, t_glist *glist)
 {
     t_my_numbox *x = (t_my_numbox *)client;
     if(glist_isvisible(glist))
     {
+        t_canvas *canvas = glist_getcanvas(glist);
+        char tag[128];
+        sprintf(tag, "%lxNUMBER", x);
         if(x->x_gui.x_fsf.x_change)
         {
             if(x->x_buf[0])
@@ -144,239 +247,34 @@ static void my_numbox_draw_update(t_gobj *client, t_glist *glist)
                 x->x_buf[sl+1] = 0;
                 if(sl >= x->x_numwidth)
                     cp += sl - x->x_numwidth + 1;
-                sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
-                         glist_getcanvas(glist), x,
-                         IEM_GUI_COLOR_EDITED, cp);
+                pdgui_vmess(0, "crs rk rs", canvas, "itemconfigure", tag,
+                    "-fill", IEM_GUI_COLOR_EDITED, "-text", cp);
                 x->x_buf[sl] = 0;
             }
             else
             {
                 my_numbox_ftoa(x);
-                sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
-                         glist_getcanvas(glist), x,
-                         IEM_GUI_COLOR_EDITED, x->x_buf);
+                pdgui_vmess(0, "crs rk rs", canvas, "itemconfigure", tag,
+                    "-fill", IEM_GUI_COLOR_EDITED, "-text", x->x_buf);
                 x->x_buf[0] = 0;
             }
         }
         else
         {
             my_numbox_ftoa(x);
-            sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
-                     glist_getcanvas(glist), x,
-                     (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol),
-                     x->x_buf);
+                pdgui_vmess(0, "crs rk rs", canvas, "itemconfigure", tag,
+                    "-fill", (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol),
+                    "-text", x->x_buf);
             x->x_buf[0] = 0;
         }
     }
 }
 
-static void my_numbox_draw_new(t_my_numbox *x, t_glist *glist)
+/* widget helper functions */
+
+static void my_numbox_tick_wait(t_my_numbox *x)
 {
-    int xpos = text_xpix(&x->x_gui.x_obj, glist);
-    int ypos = text_ypix(&x->x_gui.x_obj, glist);
-    int w = x->x_gui.x_w, half = x->x_gui.x_h/2;
-    int d = IEMGUI_ZOOM(x) + x->x_gui.x_h/(34*IEMGUI_ZOOM(x));
-    int corner = x->x_gui.x_h/4;
-    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
-    t_canvas *canvas = glist_getcanvas(glist);
-
-    sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d %d %d %d %d %d %d "
-             "-width %d -outline #%06x -fill #%06x -tags %lxBASE1\n",
-             canvas,
-             xpos, ypos,
-             xpos + w - corner, ypos,
-             xpos + w, ypos + corner,
-             xpos + w, ypos + x->x_gui.x_h,
-             xpos, ypos + x->x_gui.x_h,
-             xpos, ypos,
-             IEMGUI_ZOOM(x), IEM_GUI_COLOR_NORMAL, x->x_gui.x_bcol, x);
-    sys_vgui(".x%lx.c create line %d %d %d %d %d %d -width %d -fill #%06x -tags %lxBASE2\n",
-             canvas,
-             xpos + IEMGUI_ZOOM(x), ypos + IEMGUI_ZOOM(x),
-             xpos + half, ypos + half,
-             xpos + IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - IEMGUI_ZOOM(x),
-             IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
-    if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
-             canvas,
-             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
-             xpos + iow, ypos + x->x_gui.x_h,
-             x, 0);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
-             canvas,
-             xpos, ypos,
-             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
-             x, 0);
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
-             canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
-             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x),
-             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
-             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
-             x->x_gui.x_lcol, x);
-    my_numbox_ftoa(x);
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w -font {{%s} -%d %s} -fill #%06x -tags %lxNUMBER\n",
-             canvas, xpos + half + 2*IEMGUI_ZOOM(x), ypos + half + d,
-             x->x_buf, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x),
-             sys_fontweight, (x->x_gui.x_fsf.x_change ? IEM_GUI_COLOR_EDITED : x->x_gui.x_fcol), x);
-}
-
-static void my_numbox_draw_move(t_my_numbox *x, t_glist *glist)
-{
-    int xpos = text_xpix(&x->x_gui.x_obj, glist);
-    int ypos = text_ypix(&x->x_gui.x_obj, glist);
-    int w = x->x_gui.x_w, half = x->x_gui.x_h/2;
-    int d = IEMGUI_ZOOM(x) + x->x_gui.x_h / (34 * IEMGUI_ZOOM(x));
-    int corner = x->x_gui.x_h/4;
-    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
-    t_canvas *canvas = glist_getcanvas(glist);
-
-    sys_vgui(".x%lx.c coords %lxBASE1 %d %d %d %d %d %d %d %d %d %d %d %d\n",
-             canvas, x,
-             xpos, ypos,
-             xpos + w - corner, ypos,
-             xpos + w, ypos + corner,
-             xpos + w, ypos + x->x_gui.x_h,
-             xpos, ypos + x->x_gui.x_h,
-             xpos, ypos);
-    sys_vgui(".x%lx.c coords %lxBASE2 %d %d %d %d %d %d\n",
-             canvas, x,
-             xpos + IEMGUI_ZOOM(x), ypos + IEMGUI_ZOOM(x),
-             xpos + half, ypos + half,
-             xpos + IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - IEMGUI_ZOOM(x));
-    if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
-             canvas, x, 0,
-             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
-             xpos + iow, ypos + x->x_gui.x_h);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
-             canvas, x, 0,
-             xpos, ypos,
-             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x,
-             xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
-             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x));
-    sys_vgui(".x%lx.c coords %lxNUMBER %d %d\n",
-             canvas, x, xpos + half + 2*IEMGUI_ZOOM(x), ypos + half + d);
-}
-
-static void my_numbox_draw_erase(t_my_numbox* x, t_glist* glist)
-{
-    t_canvas *canvas = glist_getcanvas(glist);
-
-    sys_vgui(".x%lx.c delete %lxBASE1\n", canvas, x);
-    sys_vgui(".x%lx.c delete %lxBASE2\n", canvas, x);
-    sys_vgui(".x%lx.c delete %lxLABEL\n", canvas, x);
-    sys_vgui(".x%lx.c delete %lxNUMBER\n", canvas, x);
-    if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
-}
-
-static void my_numbox_draw_config(t_my_numbox* x, t_glist* glist)
-{
-    t_canvas *canvas = glist_getcanvas(glist);
-
-    sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
-             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol),
-             strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name:"");
-    sys_vgui(".x%lx.c itemconfigure %lxNUMBER -font {{%s} -%d %s} -fill #%06x \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
-             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol));
-    sys_vgui(".x%lx.c itemconfigure %lxBASE1 -fill #%06x\n", canvas,
-             x, x->x_gui.x_bcol);
-    sys_vgui(".x%lx.c itemconfigure %lxBASE2 -fill #%06x\n", canvas, x,
-             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol));
-}
-
-static void my_numbox_draw_io(t_my_numbox* x,t_glist* glist, int old_snd_rcv_flags)
-{
-    int xpos = text_xpix(&x->x_gui.x_obj, glist);
-    int ypos = text_ypix(&x->x_gui.x_obj, glist);
-    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
-    t_canvas *canvas = glist_getcanvas(glist);
-
-    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able) {
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
-             canvas,
-             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
-             xpos + iow, ypos + x->x_gui.x_h,
-             x, 0);
-        /* keep these above outlet */
-        sys_vgui(".x%lx.c raise %lxLABEL %lxOUT%d\n", canvas, x, x, 0);
-        sys_vgui(".x%lx.c raise %lxNUMBER %lxLABEL\n", canvas, x, x);
-    }
-    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
-    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able) {
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
-             canvas,
-             xpos, ypos,
-             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
-             x, 0);
-        /* keep these above inlet */
-        sys_vgui(".x%lx.c raise %lxLABEL %lxIN%d\n", canvas, x, x, 0);
-        sys_vgui(".x%lx.c raise %lxNUMBER %lxLABEL\n", canvas, x, x);
-    }
-    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
-}
-
-static void my_numbox_draw_select(t_my_numbox *x, t_glist *glist)
-{
-    t_canvas *canvas = glist_getcanvas(glist);
-
-    if(x->x_gui.x_fsf.x_selected)
-    {
-        if(x->x_gui.x_fsf.x_change)
-        {
-            x->x_gui.x_fsf.x_change = 0;
-            clock_unset(x->x_clock_reset);
-            x->x_buf[0] = 0;
-            sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
-        }
-        sys_vgui(".x%lx.c itemconfigure %lxBASE1 -outline #%06x\n",
-            canvas, x, IEM_GUI_COLOR_SELECTED);
-        sys_vgui(".x%lx.c itemconfigure %lxBASE2 -fill #%06x\n",
-            canvas, x, IEM_GUI_COLOR_SELECTED);
-        sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n",
-            canvas, x, IEM_GUI_COLOR_SELECTED);
-        sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x\n",
-            canvas, x, IEM_GUI_COLOR_SELECTED);
-    }
-    else
-    {
-        sys_vgui(".x%lx.c itemconfigure %lxBASE1 -outline #%06x\n",
-            canvas, x, IEM_GUI_COLOR_NORMAL);
-        sys_vgui(".x%lx.c itemconfigure %lxBASE2 -fill #%06x\n",
-            canvas, x, x->x_gui.x_fcol);
-        sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n",
-            canvas, x, x->x_gui.x_lcol);
-        sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x\n",
-            canvas, x, x->x_gui.x_fcol);
-    }
-}
-
-void my_numbox_draw(t_my_numbox *x, t_glist *glist, int mode)
-{
-    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
-        sys_queuegui(x, glist, my_numbox_draw_update);
-    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
-        my_numbox_draw_move(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_NEW)
-        my_numbox_draw_new(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
-        my_numbox_draw_select(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
-        my_numbox_draw_erase(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
-        my_numbox_draw_config(x, glist);
-    else if(mode >= IEM_GUI_DRAW_MODE_IO)
-        my_numbox_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
+    sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
 }
 
 /* ------------------------ nbx widgetbehaviour----------------------------- */
@@ -402,7 +300,6 @@ static void my_numbox_save(t_gobj *z, t_binbuf *b)
     if(x->x_gui.x_fsf.x_change)
     {
         x->x_gui.x_fsf.x_change = 0;
-        clock_unset(x->x_clock_reset);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     }
     binbuf_addv(b, "ssiisiiffiisssiiiisssfi", gensym("#X"), gensym("obj"),
@@ -459,34 +356,19 @@ int my_numbox_check_minmax(t_my_numbox *x, double min, double max)
 static void my_numbox_properties(t_gobj *z, t_glist *owner)
 {
     t_my_numbox *x = (t_my_numbox *)z;
-    char buf[800];
-    t_symbol *srl[3];
 
-    iemgui_properties(&x->x_gui, srl);
     if(x->x_gui.x_fsf.x_change)
     {
         x->x_gui.x_fsf.x_change = 0;
-        clock_unset(x->x_clock_reset);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     }
-    sprintf(buf, "pdtk_iemgui_dialog %%s |nbx| \
-            -------dimensions(digits)(pix):------- %d %d width: %d %d height: \
-            -----------output-range:----------- %g min: %g max: %d \
-            %d lin log %d %d log-height: %d \
-            %s %s \
-            %s %d %d \
-            %d %d \
-            #%06x #%06x #%06x\n",
-            x->x_numwidth, MINDIGITS, x->x_gui.x_h/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
-            x->x_min, x->x_max, 0,/*no_schedule*/
-            x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, -1,
-                x->x_log_height, /*no multi, but iem-characteristic*/
-            srl[0]->s_name, srl[1]->s_name,
-            srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
-            x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
-            0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol,
-                0xffffff & x->x_gui.x_lcol);
-    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
+    iemgui_new_dialog(x, &x->x_gui, "nbx",
+                      x->x_numwidth, MINDIGITS,
+                      x->x_gui.x_h/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
+                      x->x_min, x->x_max,
+                      0,
+                      x->x_lin0_log1, "linear", "logarithmic",
+                      1, -1, x->x_log_height);
 }
 
 static void my_numbox_bang(t_my_numbox *x)
@@ -536,10 +418,7 @@ static void my_numbox_dialog(t_my_numbox *x, t_symbol *s, int argc,
      my_numbox_bang(x);*/
     my_numbox_check_minmax(x, min, max);
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
-    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
-    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
-    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
-    canvas_fixlinesfor(x->x_gui.x_glist, (t_text*)x);
+    iemgui_size(x, &x->x_gui);
 }
 
 static void my_numbox_motion(t_my_numbox *x, t_floatarg dx, t_floatarg dy,
@@ -558,7 +437,6 @@ static void my_numbox_motion(t_my_numbox *x, t_floatarg dx, t_floatarg dy,
     my_numbox_clip(x);
     sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     my_numbox_bang(x);
-    clock_unset(x->x_clock_reset);
 }
 
 static void my_numbox_click(t_my_numbox *x, t_floatarg xpos, t_floatarg ypos,
@@ -585,14 +463,11 @@ static int my_numbox_newclick(t_gobj *z, struct _glist *glist,
         {
             clock_delay(x->x_clock_wait, 50);
             x->x_gui.x_fsf.x_change = 1;
-            clock_delay(x->x_clock_reset, 3000);
-
             x->x_buf[0] = 0;
         }
         else
         {
             x->x_gui.x_fsf.x_change = 0;
-            clock_unset(x->x_clock_reset);
             x->x_buf[0] = 0;
             sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
         }
@@ -608,7 +483,8 @@ static void my_numbox_set(t_my_numbox *x, t_floatarg f)
     if (memcmp(&ftocompare, &x->x_val, sizeof(ftocompare)))
     {
         x->x_val = ftocompare;
-        my_numbox_clip(x);
+        if (pd_compatibilitylevel < 53)
+            my_numbox_clip(x);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     }
 }
@@ -648,6 +524,7 @@ static void my_numbox_size(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
     }
     my_numbox_calc_fontwidth(x);
     iemgui_size((void *)x, &x->x_gui);
+    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
 }
 
 static void my_numbox_delta(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
@@ -736,7 +613,6 @@ static void my_numbox_key(void *z, t_symbol *keysym, t_floatarg fkey)
     if(c == 0)
     {
         x->x_gui.x_fsf.x_change = 0;
-        clock_unset(x->x_clock_reset);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
         return;
     }
@@ -761,15 +637,16 @@ static void my_numbox_key(void *z, t_symbol *keysym, t_floatarg fkey)
     }
     else if((c == '\n') || (c == 13))
     {
-        x->x_val = atof(x->x_buf);
-        x->x_buf[0] = 0;
-        x->x_gui.x_fsf.x_change = 0;
-        clock_unset(x->x_clock_reset);
-        my_numbox_clip(x);
+        if(x->x_buf[0])
+        {
+            x->x_val = atof(x->x_buf);
+            x->x_buf[0] = 0;
+            if (pd_compatibilitylevel < 53)
+                my_numbox_clip(x);
+            sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
+        }
         my_numbox_bang(x);
-        sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     }
-    clock_delay(x->x_clock_reset, 3000);
 }
 
 static void my_numbox_list(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
@@ -786,16 +663,14 @@ static void my_numbox_list(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 
 static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
 {
-    t_my_numbox *x = (t_my_numbox *)pd_new(my_numbox_class);
-    int w = 5, h = 14;
-    int lilo = 0, ldx = 0, ldy = -8;
-    int fs = 10;
+    t_my_numbox *x = (t_my_numbox *)iemgui_new(my_numbox_class);
+    int w = 5, h = 14 * IEM_GUI_DEFAULTSIZE_SCALE;
+    int lilo = 0, ldx = 0, ldy = -8 * IEM_GUI_DEFAULTSIZE_SCALE;
+    int fs = x->x_gui.x_fontsize;
     int log_height = 256;
     double min = -1.0e+37, max = 1.0e+37, v = 0.0;
 
-    x->x_gui.x_bcol = 0xFCFCFC;
-    x->x_gui.x_fcol = 0x00;
-    x->x_gui.x_lcol = 0x00;
+    IEMGUI_SETDRAWFUNCTIONS(x, my_numbox);
 
     if((argc >= 17)&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
        &&IS_A_FLOAT(argv,2)&&IS_A_FLOAT(argv,3)
@@ -825,10 +700,8 @@ static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
     {
         log_height = (int)atom_getfloatarg(17, argc, argv);
     }
-    x->x_gui.x_draw = (t_iemfunptr)my_numbox_draw;
-    x->x_gui.x_fsf.x_snd_able = 1;
-    x->x_gui.x_fsf.x_rcv_able = 1;
-    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
+    x->x_gui.x_fsf.x_snd_able = (0 != x->x_gui.x_snd);
+    x->x_gui.x_fsf.x_rcv_able = (0 != x->x_gui.x_rcv);
     if(x->x_gui.x_isa.x_loadinit)
         x->x_val = v;
     else
@@ -838,10 +711,6 @@ static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
     if(log_height < 10)
         log_height = 10;
     x->x_log_height = log_height;
-    if(!strcmp(x->x_gui.x_snd->s_name, "empty"))
-        x->x_gui.x_fsf.x_snd_able = 0;
-    if(!strcmp(x->x_gui.x_rcv->s_name, "empty"))
-        x->x_gui.x_fsf.x_rcv_able = 0;
     if(x->x_gui.x_fsf.x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
     else if(x->x_gui.x_fsf.x_font_style == 2) strcpy(x->x_gui.x_font, "times");
     else { x->x_gui.x_fsf.x_font_style = 0;
@@ -850,9 +719,7 @@ static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
         pd_bind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
     x->x_gui.x_ldx = ldx;
     x->x_gui.x_ldy = ldy;
-    if(fs < MINFONT)
-        fs = MINFONT;
-    x->x_gui.x_fontsize = fs;
+    x->x_gui.x_fontsize = (fs < MINFONT)?MINFONT:fs;
     if(w < MINDIGITS)
         w = MINDIGITS;
     x->x_numwidth = w;
@@ -862,7 +729,6 @@ static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
     x->x_buf[0] = 0;
     my_numbox_check_minmax(x, min, max);
     iemgui_verify_snd_ne_rcv(&x->x_gui);
-    x->x_clock_reset = clock_new(x, (t_method)my_numbox_tick_reset);
     x->x_clock_wait = clock_new(x, (t_method)my_numbox_tick_wait);
     x->x_gui.x_fsf.x_change = 0;
     iemgui_newzoom(&x->x_gui);
@@ -875,9 +741,8 @@ static void my_numbox_free(t_my_numbox *x)
 {
     if(x->x_gui.x_fsf.x_rcv_able)
         pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
-    clock_free(x->x_clock_reset);
     clock_free(x->x_clock_wait);
-    gfxstub_deleteforkey(x);
+    pdgui_stub_deleteforkey(x);
 }
 
 void g_numbox_setup(void)
@@ -936,7 +801,6 @@ void g_numbox_setup(void)
     my_numbox_widgetbehavior.w_visfn =        iemgui_vis;
     my_numbox_widgetbehavior.w_clickfn =      my_numbox_newclick;
     class_setwidget(my_numbox_class, &my_numbox_widgetbehavior);
-    class_sethelpsymbol(my_numbox_class, gensym("nbx"));
     class_setsavefn(my_numbox_class, my_numbox_save);
     class_setpropertiesfn(my_numbox_class, my_numbox_properties);
 }
